/*

** type.d

** Copyright (C) 2003 Helmut Leitner

** Donated to Digital Mars.

** Released under the GNU GPL, see LICENSE.TXT

** and http://www.gnu.org/copyleft/gpl.html

*/



module venus.type;

import venus.all;



extern (C) {

   long atol(char *);

}



alias short error;



struct TypeMethods {

    uint (*hash)(void *p);

    int  (*retsize)();

    int  (*equ )(void *p1,void *p2);

    int  (*cmp )(void *p1,void *p2);

    void (*swap)(void *p1,void *p2);

    int  (*gets)(void *p1,char *buf);

    int  (*puts)(void *p1,char *buf);

    int  (*getf)(void *p1,char *buf,int w,int d,int opt);

}



typedef uint (*cast_method_hash)(void *);

typedef  int (*cast_method_retsize)() ;

typedef  int (*cast_method_equ )(void *p1,void *p2);

typedef  int (*cast_method_cmp )(void *p1,void *p2);

typedef void (*cast_method_swap)(void *p1,void *p2);

typedef  int (*cast_method_gets)(void *p1,char *buf);

typedef  int (*cast_method_getf)(void *p1,char *buf,int w,int d,int opt);

typedef  int (*cast_method_puts)(void *p1,char *buf);



struct Tpi {

    void *pv;

    TypeMethods *ptm;

}



//==========



int char_retsize() {

    return char.size;

}



uint char_hash(char *p) {

    return *(uint *)p;

}



int char_cmp(char *p1, char *p2)

{

    return *p1 - *p2;

}



int char_equ(char *p1, char *p2)

{

    return (*p1 == *p2);

}



void char_swap(char *p1, char *p2)

{

    char t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int char_gets(char *p,char *buf)

{

    return sprintf(buf,"%c",*p);

}



int char_puts(char *p,char *buf)

{

    *p = buf[0];

    return 0;

}



int char_getf(char *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%c",*p);

}



TypeMethods char_Methods = {

    hash: (cast_method_hash) &char_hash,

    retsize: (cast_method_retsize) &char_retsize,

    equ : (cast_method_equ ) &char_equ ,

    cmp : (cast_method_cmp ) &char_cmp ,

    swap: (cast_method_swap) &char_swap,

    gets: (cast_method_gets) &char_gets,

    puts: (cast_method_puts) &char_puts,

    getf: (cast_method_getf) &char_getf

};



Tpi TypeRetTpi(inout char t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &char_Methods;

    return ret;

}



//==========



int byte_retsize() {

    return byte.size;

}



uint byte_hash(byte *p) {

    return *(uint *)p;

}



int byte_cmp(byte *p1, byte *p2)

{

    return *p1 - *p2;

}



int byte_equ(byte *p1, byte *p2)

{

    return (*p1 == *p2);

}



void byte_swap(byte *p1, byte *p2)

{

    byte t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int byte_gets(byte *p,char *buf)

{

    return sprintf(buf,"%d",*p);

}



int byte_getf(byte *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*d",w,d,*p);

}



int byte_puts(byte *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



TypeMethods byte_Methods = {

    hash: (cast_method_hash) &byte_hash,

    retsize: (cast_method_retsize) &byte_retsize,

    equ : (cast_method_equ ) &byte_equ ,

    cmp : (cast_method_cmp ) &byte_cmp ,

    swap: (cast_method_swap) &byte_swap,

    gets: (cast_method_gets) &byte_gets,

    puts: (cast_method_puts) &byte_puts,

    getf: (cast_method_getf) &byte_getf

};



Tpi TypeRetTpi(inout byte t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &byte_Methods;

    return ret;

}



//==========



int ubyte_retsize() {

    return ubyte.size;

}



uint ubyte_hash(ubyte *p) {

    return *(uint *)p;

}



int ubyte_cmp(ubyte *p1, ubyte *p2)

{

    return *p1 - *p2;

}



int ubyte_equ(ubyte *p1, ubyte *p2)

{

    return (*p1 == *p2);

}



void ubyte_swap(ubyte *p1, ubyte *p2)

{

    ubyte t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int ubyte_gets(ubyte *p,char *buf)

{

    return sprintf(buf,"%d",*p);

}



int ubyte_getf(ubyte *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*d",w,d,*p);

}



int ubyte_puts(ubyte *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



TypeMethods ubyte_Methods = {

    hash: (cast_method_hash) &ubyte_hash,

    retsize: (cast_method_retsize) &ubyte_retsize,

    equ : (cast_method_equ ) &ubyte_equ ,

    cmp : (cast_method_cmp ) &ubyte_cmp ,

    swap: (cast_method_swap) &ubyte_swap,

    gets: (cast_method_gets) &ubyte_gets,

    puts: (cast_method_puts) &ubyte_puts,

    getf: (cast_method_getf) &ubyte_getf

};



Tpi TypeRetTpi(inout ubyte t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &ubyte_Methods;

    return ret;

}



//==========



int short_retsize() {

    return short.size;

}



uint short_hash(short *p) {

    return *(uint *)p;

}



int short_cmp(short *p1, short *p2)

{

    return *p1 - *p2;

}



int short_equ(short *p1, short *p2)

{

    return (*p1 == *p2);

}



void short_swap(short *p1, short *p2)

{

    short t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int short_gets(short *p,char *buf)

{

    return sprintf(buf,"%d",*p);

}



int short_puts(short *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



int short_getf(short *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*d",w,d,*p);

}



TypeMethods short_Methods = {

    hash: (cast_method_hash) &short_hash,

    retsize: (cast_method_retsize) &short_retsize,

    equ : (cast_method_equ ) &short_equ ,

    cmp : (cast_method_cmp ) &short_cmp ,

    swap: (cast_method_swap) &short_swap,

    gets: (cast_method_gets) &short_gets,

    puts: (cast_method_puts) &short_puts,

    getf: (cast_method_getf) &short_getf

};



Tpi TypeRetTpi(inout short t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &short_Methods;

    return ret;

}



//==========



int ushort_retsize() {

    return ushort.size;

}



uint ushort_hash(ushort *p) {

    return *(uint *)p;

}



int ushort_cmp(ushort *p1, ushort *p2)

{

    return *p1 - *p2;

}



int ushort_equ(ushort *p1, ushort *p2)

{

    return (*p1 == *p2);

}



void ushort_swap(ushort *p1, ushort *p2)

{

    ushort t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int ushort_gets(ushort *p,char *buf)

{

    return sprintf(buf,"%d",*p);

}



int ushort_puts(ushort *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



int ushort_getf(ushort *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*d",w,d,*p);

}



TypeMethods ushort_Methods = {

    hash: (cast_method_hash) &ushort_hash,

    retsize: (cast_method_retsize) &ushort_retsize,

    equ : (cast_method_equ ) &ushort_equ ,

    cmp : (cast_method_cmp ) &ushort_cmp ,

    swap: (cast_method_swap) &ushort_swap,

    gets: (cast_method_gets) &ushort_gets,

    puts: (cast_method_puts) &ushort_puts,

    getf: (cast_method_getf) &ushort_getf

};



Tpi TypeRetTpi(inout ushort t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &ushort_Methods;

    return ret;

}



//==========



int int_retsize() {

    return int.size;

}



uint int_hash(int *p) {

    return *(uint *)p;

}



int int_cmp(int *p1, int *p2)

{

    return *p1 - *p2;

}



int int_equ(int *p1, int *p2)

{

    return (*p1 == *p2);

}



void int_swap(int *p1, int *p2)

{

    int t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int int_gets(int *p,char *buf)

{

    return sprintf(buf,"%d",*p);

}



int int_puts(int *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



int int_getf(int *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*d",w,d,*p);

}



TypeMethods int_Methods = {

    hash: (cast_method_hash) &int_hash,

    retsize: (cast_method_retsize) &int_retsize,

    equ : (cast_method_equ ) &int_equ ,

    cmp : (cast_method_cmp ) &int_cmp ,

    swap: (cast_method_swap) &int_swap,

    gets: (cast_method_gets) &int_gets,

    puts: (cast_method_puts) &int_puts,

    getf: (cast_method_getf) &int_getf

};



Tpi TypeRetTpi(inout int t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &int_Methods;

    return ret;

}



//==========



uint uint_retsize() {

    return uint.size;

}



uint uint_hash(uint *p) {

    return *(uint *)p;

}



uint uint_cmp(uint *p1, uint *p2)

{

    return *p1 - *p2;

}



uint uint_equ(uint *p1, uint *p2)

{

    return (*p1 == *p2);

}



void uint_swap(uint *p1, uint *p2)

{

    uint t= *p1;

    *p1 = *p2;

    *p2 = t;

}



int uint_gets(uint *p,char *buf)

{

    return sprintf(buf,"%u",*p);

}



int uint_puts(uint *p,char *buf)

{

    *p = atol(buf);

    return 0;

}



uint uint_getf(uint *p,char *buf,uint w,uint d)

{

    return sprintf(buf,"%*.*u",w,d,*p);

}



TypeMethods uint_Methods = {

    hash: (cast_method_hash) &uint_hash,

    retsize: (cast_method_retsize) &uint_retsize,

    equ : (cast_method_equ ) &uint_equ ,

    cmp : (cast_method_cmp ) &uint_cmp ,

    swap: (cast_method_swap) &uint_swap,

    gets: (cast_method_gets) &uint_gets,

    puts: (cast_method_puts) &uint_puts,

    getf: (cast_method_getf) &uint_getf

};



Tpi TypeRetTpi(inout uint t) {

    Tpi ret;

    ret.pv=&t; // !!! no cast (void *)&t

    ret.ptm= &uint_Methods;

    return ret;

}



//==========



int long_retsize() {

    return long.size;

}



uint long_hash(long *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int long_cmp(long *p1, long *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int long_equ(long *p1, long *p2)

{

    return (*p1 == *p2);

}



int long_gets(long *p,char *buf)

{

    return sprintf(buf,"%lld",*p);

}



int long_puts(long *p,char *buf)

{

    sscanf(buf,"%lld",p);

    return 0;

}



int long_getf(long *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*lld",w,d,*p);

}



void long_swap(long *p1, long *p2)

{

    long t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods long_Methods = {

    hash: (cast_method_hash) &long_hash,

    retsize: (cast_method_retsize) &long_retsize,

    equ : (cast_method_equ ) &long_equ ,

    cmp : (cast_method_cmp ) &long_cmp ,

    swap: (cast_method_swap) &long_swap,

    gets: (cast_method_gets) &long_gets,

    puts: (cast_method_puts) &long_puts,

    getf: (cast_method_getf) &long_getf

};



Tpi TypeRetTpi(inout long t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &long_Methods;

    return ret;

}



//==========



int ulong_retsize() {

    return ulong.size;

}



uint ulong_hash(ulong *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int ulong_cmp(ulong *p1, ulong *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int ulong_equ(ulong *p1, ulong *p2)

{

    return (*p1 == *p2);

}



int ulong_gets(ulong *p,char *buf)

{

    return sprintf(buf,"%llu",*p);

}



int ulong_puts(ulong *p,char *buf)

{

    sscanf(buf,"%llu",p);

    return 0;

}



int ulong_getf(ulong *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*llu",w,d,*p);

}



void ulong_swap(ulong *p1, ulong *p2)

{

    ulong t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods ulong_Methods = {

    hash: (cast_method_hash) &ulong_hash,

    retsize: (cast_method_retsize) &ulong_retsize,

    equ : (cast_method_equ ) &ulong_equ ,

    cmp : (cast_method_cmp ) &ulong_cmp ,

    swap: (cast_method_swap) &ulong_swap,

    gets: (cast_method_gets) &ulong_gets,

    puts: (cast_method_puts) &ulong_puts,

    getf: (cast_method_getf) &ulong_getf

};



Tpi TypeRetTpi(inout ulong t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &ulong_Methods;

    return ret;

}



//==========



int float_retsize() {

    return float.size;

}



uint float_hash(float *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int float_cmp(float *p1, float *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int float_equ(float *p1, float *p2)

{

    return (*p1 == *p2);

}



int float_gets(float *p,char *buf)

{

    return sprintf(buf,"%f",*p);

}



int float_puts(float *p,char *buf)

{

    *p = math.atof(buf);

    return 0;

}



int float_getf(float *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*f",w,d,*p);

}



void float_swap(float *p1, float *p2)

{

    float t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods float_Methods = {

    hash: (cast_method_hash) &float_hash,

    retsize: (cast_method_retsize) &float_retsize,

    equ : (cast_method_equ ) &float_equ ,

    cmp : (cast_method_cmp ) &float_cmp ,

    swap: (cast_method_swap) &float_swap,

    gets: (cast_method_gets) &float_gets,

    puts: (cast_method_puts) &float_puts,

    getf: (cast_method_getf) &float_getf

};



Tpi TypeRetTpi(inout float t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &float_Methods;

    return ret;

}



//==========



int double_retsize() {

    return double.size;

}



uint double_hash(double *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int double_cmp(double *p1, double *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int double_equ(double *p1, double *p2)

{

    return (*p1 == *p2);

}



int double_gets(double *p,char *buf)

{

    return sprintf(buf,"%f",*p);

}



int double_puts(double *p,char *buf)

{

    *p = math.atof(buf);

    return 0;

}



int double_getf(double *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*f",w,d,*p);

}



void double_swap(double *p1, double *p2)

{

    double t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods double_Methods = {

    hash: (cast_method_hash) &double_hash,

    retsize: (cast_method_retsize) &double_retsize,

    equ : (cast_method_equ ) &double_equ ,

    cmp : (cast_method_cmp ) &double_cmp ,

    swap: (cast_method_swap) &double_swap,

    gets: (cast_method_gets) &double_gets,

    puts: (cast_method_puts) &double_puts,

    getf: (cast_method_getf) &double_getf

};



Tpi TypeRetTpi(inout double t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &double_Methods;

    return ret;

}



//==========



int ifloat_retsize() {

    return ifloat.size;

}



uint ifloat_hash(ifloat *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int ifloat_cmp(ifloat *p1, ifloat *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int ifloat_equ(ifloat *p1, ifloat *p2)

{

    return (*p1 == *p2);

}



int ifloat_gets(ifloat *p,char *buf)

{

    return sprintf(buf,"%fi",*p);

}



int ifloat_puts(ifloat *p,char *buf)

{

    *p = math.atof(buf);

    return 0;

}



int ifloat_getf(ifloat *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*fi",w,d,*p);

}



void ifloat_swap(ifloat *p1, ifloat *p2)

{

    ifloat t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods ifloat_Methods = {

    hash: (cast_method_hash) &ifloat_hash,

    retsize: (cast_method_retsize) &ifloat_retsize,

    equ : (cast_method_equ ) &ifloat_equ ,

    cmp : (cast_method_cmp ) &ifloat_cmp ,

    swap: (cast_method_swap) &ifloat_swap,

    gets: (cast_method_gets) &ifloat_gets,

    puts: (cast_method_puts) &ifloat_puts,

    getf: (cast_method_getf) &ifloat_getf

};



Tpi TypeRetTpi(inout ifloat t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &ifloat_Methods;

    return ret;

}



//==========



int idouble_retsize() {

    return idouble.size;

}



uint idouble_hash(idouble *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int idouble_cmp(idouble *p1, idouble *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int idouble_equ(idouble *p1, idouble *p2)

{

    return (*p1 == *p2);

}



int idouble_gets(idouble *p,char *buf)

{

    return sprintf(buf,"%fi",*p);

}



int idouble_puts(idouble *p,char *buf)

{

    *p = math.atof(buf);

    return 0;

}



int idouble_getf(idouble *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*fi",w,d,*p);

}



void idouble_swap(idouble *p1, idouble *p2)

{

    idouble t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods idouble_Methods = {

    hash: (cast_method_hash) &idouble_hash,

    retsize: (cast_method_retsize) &idouble_retsize,

    equ : (cast_method_equ ) &idouble_equ ,

    cmp : (cast_method_cmp ) &idouble_cmp ,

    swap: (cast_method_swap) &idouble_swap,

    gets: (cast_method_gets) &idouble_gets,

    puts: (cast_method_gets) &idouble_puts,

    getf: (cast_method_getf) &idouble_getf

};



Tpi TypeRetTpi(inout idouble t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &idouble_Methods;

    return ret;

}



//==========



int cfloat_retsize() {

    return cfloat.size;

}



uint cfloat_hash(cfloat *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int cfloat_cmp(cfloat *p1, cfloat *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int cfloat_equ(cfloat *p1, cfloat *p2)

{

    return (*p1 == *p2);

}



int cfloat_gets(cfloat *p,char *buf)

{

    return sprintf(buf,"%f + %fi",(*p).re,(*p).im);

}



int StrGetCdouble(char *buf,inout cdouble cd)

{

   char buf1[256];

   char buf2[256];

   double re=0.0;

   idouble im=0.0;

   double d;

   int sign=0;

   char *p=strchr(buf,'+');





   if(p) {

      memset.memcpy(&buf1[0],buf,(uint)(p-buf));

      strcpy(&buf2[0],p+1);

   } else {

      strcpy(&buf1[0],buf);

      buf2[0]=0;

   }

   StrStrip(buf1);

   StrStrip(buf2);



   d=math.atof(buf1);

   p=strchr(&buf1[0],'i');

   if(p) {

      *p=0;

      im=d;

   } else {

      re=d;

   }



   p=strchr(&buf2[0],'i');

   if(p) {

      *p=0;

   }

   d=math.atof(buf2);

   if(sign) {

      d= -d;

   }

   memset.memcpy(&im,&d,im.size);



   cd = re + im;



   return 0;

}



int cfloat_puts(cfloat *p,char *buf)

{

    cdouble cd;



    int err=StrGetCdouble(buf,cd);

    if(err) {

       *p=0.0;

    } else {

       *p = cd;

    }

    return err;

}



int cfloat_getf(cfloat *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*f + %*.*fi",w,d,(*p).re,w,d,(*p).im);

}



void cfloat_swap(cfloat *p1, cfloat *p2)

{

    cfloat t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods cfloat_Methods = {

    hash: (cast_method_hash) &cfloat_hash,

    retsize: (cast_method_retsize) &cfloat_retsize,

    equ : (cast_method_equ ) &cfloat_equ ,

    cmp : (cast_method_cmp ) &cfloat_cmp ,

    swap: (cast_method_swap) &cfloat_swap,

    gets: (cast_method_gets) &cfloat_gets,

    puts: (cast_method_puts) &cfloat_puts,

    getf: (cast_method_getf) &cfloat_getf

};



Tpi TypeRetTpi(inout cfloat t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &cfloat_Methods;

    return ret;

}



//==========



int cdouble_retsize() {

    return cdouble.size;

}



uint cdouble_hash(cdouble *p) {

    return ((uint *)p)[0] + ((uint *)p)[1];

}



int cdouble_cmp(cdouble *p1, cdouble *p2)

{

   return *p1 < *p2 ? -1 : *p1 > *p2 ? 1 : 0;

}



int cdouble_equ(cdouble *p1, cdouble *p2)

{

    return (*p1 == *p2);

}



int cdouble_gets(cdouble *p,char *buf)

{

    return sprintf(buf,"%f + %fi",(*p).re,(*p).im);

}



int cdouble_puts(cdouble *p,char *buf)

{

    cdouble cd;



    int err=StrGetCdouble(buf,cd);

    if(err) {

       *p=0.0;

    } else {

       *p = cd;

    }

    return err;

}



int cdouble_getf(cdouble *p,char *buf,int w,int d,int opt)

{

    return sprintf(buf,"%*.*f + %*.*fi",w,d,(*p).re,w,d,(*p).im);

}



void cdouble_swap(cdouble *p1, cdouble *p2)

{

    cdouble t= *p1;

    *p1 = *p2;

    *p2 = t;

}



TypeMethods cdouble_Methods = {

    hash: (cast_method_hash) &cdouble_hash,

    retsize: (cast_method_retsize) &cdouble_retsize,

    equ : (cast_method_equ ) &cdouble_equ ,

    cmp : (cast_method_cmp ) &cdouble_cmp ,

    swap: (cast_method_swap) &cdouble_swap,

    gets: (cast_method_gets) &cdouble_gets,

    puts: (cast_method_puts) &cdouble_puts,

    getf: (cast_method_getf) &cdouble_getf

};



Tpi TypeRetTpi(inout cdouble t) {

    Tpi ret;

    ret.pv=&t;

    ret.ptm= &cdouble_Methods;

    return ret;

}



